//
//  MADEngine.h
//  MADCore
//
//  Created by JustinLu on 9/13/17.
//  Copyright © 2017 Chengming Lu. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "WXLog.h"
#import "CSIICrashReporterConfig.h"
#import "MADCoreConfig.h"

@class WorkerApiResponse;
@class WorkerRequest;

typedef enum{
    kRiskStubServiceCheckTypeUdid,
    kRiskStubServiceCheckTypeDevInfo,
    kRiskStubServiceCheckTypeApkInfo,
    kRiskStubServiceCheckTypeStartInfo,
    kRiskStubServiceCheckTypeStart,
    kRiskStubServiceCheckTypeCrash,
    kRiskStubServiceCheckTypeEmulator,
    kRiskStubServiceCheckTypeLocation,
    kRiskStubServiceCheckTypeRoot,
    kRiskStubServiceCheckTypeHostFraud,
    kRiskStubServiceCheckTypeDevices_Reuse,
    kRiskStubServiceCheckTypeInject,
    kRiskStubServiceCheckTypeDebugger,
    kRiskStubServiceCheckTypeGamePlugin,
    kRiskStubServiceCheckTypeSpeed,
    kRiskStubServiceCheckTypeEnvCheck,
    kRiskStubServiceCheckTypeGyroscope,
    kRiskStubServiceCheckTypeMultiMessage,
    kRiskStubServiceCheckTypeScreenSharing,
    kRiskStubServiceCheckTypeUnknown
} BCESRiskStubServiceCheckType;

typedef enum {
    kRiskStubServiceActionMessage, // 只提示
    kRiskStubServiceActionMessageThenQuit, // 提示后强制退出
    kRiskStubServiceActionQuitOrMessage, // 提示后选择退出
    kRiskStubServiceActionQuit // 直接强退
} BCESRiskStubServiceAction;

@interface MADEngine : NSObject

typedef void (^MADEngineFunction)(NSDictionary *params);
typedef void (^InitEnvCallback)(void);

+ (void)registerModule:(NSString *)name withController:(Class)vc;
+ (void)registerModule:(NSString *)name withFunction:(MADEngineFunction)func;

/**
 初始化MADCore引擎
 @param callback callback用于应用层注册自己weexModule或weexComponent
 */
+ (void)initEnvironment:(InitEnvCallback)callback __attribute__((deprecated("Use -initEnvironmentWithMADCoreConfig:(MADCoreConfig *)config callback:(InitEnvCallback)callback;")));

/**
 初始化MADCore引擎
 */
+ (void)initEnvironment __attribute__((deprecated("Use -initEnvironmentWithMADCoreConfig:(MADCoreConfig *)config callback:(InitEnvCallback)callback;")));
/**
 初始化MADCore引擎
 @param level WeexSDK、MADCore、Worker共用日志级别，默认WXLogLevelWarning
 */
+ (void)initEnvironmentWithLogLevel:(WXLogLevel)level __attribute__((deprecated("Use -initEnvironmentWithMADCoreConfig:(MADCoreConfig *)config callback:(InitEnvCallback)callback;")));
/**
初始化MADCore引擎
  @param level WeexSDK、MADCore、Worker共用日志级别，默认WXLogLevelWarning
  @param callback callback用于应用层注册自己weexModule或weexComponent
 */
+ (void)initEnvironmentWithLogLevel:(WXLogLevel)level callback:(InitEnvCallback)callback __attribute__((deprecated("Use -initEnvironmentWithMADCoreConfig:(MADCoreConfig *)config callback:(InitEnvCallback)callback;")));
/**
 初始化MADCore引擎
 @param level WeexSDK、MADCore、Worker共用日志级别，默认WXLogLevelWarning
 @param proxy 代理地址
 @param callback callback用于应用层注册自己weexModule或weexComponent
 */
+ (void)initEnvironmentWithLogLevel:(WXLogLevel)level httpProxy:(NSString *)proxy callback:(InitEnvCallback)callback __attribute__((deprecated("Use -initEnvironmentWithMADCoreConfig:(MADCoreConfig *)config callback:(InitEnvCallback)callback;")));
/**
 初始化MADCore引擎
 @param level WeexSDK、MADCore共用日志级别，默认WXLogLevelWarning
 @param workerLog Worker日志开关单独设置
 @param callback callback用于应用层注册自己weexModule或weexComponent
 */
+ (void)initEnvironmentWithLogLevel:(WXLogLevel)level workerLog:(BOOL)workerLog callback:(InitEnvCallback)callback __attribute__((deprecated("Use -initEnvironmentWithMADCoreConfig:(MADCoreConfig *)config callback:(InitEnvCallback)callback;")));
/**
初始化MADCore引擎
@param level WeexSDK、MADCore共用日志级别，默认WXLogLevelWarning
@param isUse isUse 默认NO 走workerApi请求资源  YES 使用代理端口
@param callback callback用于应用层注册自己weexModule或weexComponent
*/
+ (void)initEnvironmentWithLogLevel:(WXLogLevel)level isUsePortService:(BOOL)isUse callback:(InitEnvCallback)callback __attribute__((deprecated("Use -initEnvironmentWithMADCoreConfig:(MADCoreConfig *)config callback:(InitEnvCallback)callback;")));
/**
 初始化MADCore引擎
 @param level WeexSDK、MADCore共用日志级别，默认WXLogLevelWarning
 @param workerLog Worker日志开关单独设置
 @param proxy 代理地址
 @param callback callback用于应用层注册自己weexModule或weexComponent
 */
+ (void)initEnvironmentWithLogLevel:(WXLogLevel)level workerLog:(BOOL)workerLog httpProxy:(NSString *)proxy callback:(InitEnvCallback)callback __attribute__((deprecated("Use -initEnvironmentWithMADCoreConfig:(MADCoreConfig *)config callback:(InitEnvCallback)callback;")));
/**
初始化MADCore引擎
@param level WeexSDK、MADCore共用日志级别，默认WXLogLevelWarning
@param workerLog Worker日志开关单独设置
@param proxy 代理地址
@param isUse isUse 默认NO 走workerApi请求资源  YES 使用代理端口
@param callback callback用于应用层注册自己weexModule或weexComponent
*/
+ (void)initEnvironmentWithLogLevel:(WXLogLevel)level workerLog:(BOOL)workerLog httpProxy:(NSString *)proxy isUsePortService:(BOOL)isUse callback:(InitEnvCallback)callback __attribute__((deprecated("Use +initEnvironmentWithMADCoreConfig:(MADCoreConfig *)config callback:(InitEnvCallback)callback;")));

/// 初始化MADCore引擎
/// @param config MADCoreConfig MADP配置
/// @param callback callback用于应用层注册自己weexModule或weexComponent
+ (void)initEnvironmentWithMADCoreConfig:(MADCoreConfig *)config callback:(InitEnvCallback)callback;

/**
 开启客户端埋点
 @param host 埋点日志发送的服务的地址
*/
+ (void)startBurialPointWithHost:(NSString *)host;


//开启日志采集
+ (void)initCrashReport:(CSIICrashReporterConfig *)config;
//绑定customerID
+(void)crashReporterBindCustomerID:(NSString *)customerID;
//获取dns
+(NSString *)getDNS:(NSString*)host;

/**
邦盛行为分析，无此集成无需调用
 @param url 请求地址
 @param cust_id cust_id
 */
+ (void)initCQBConfig:(NSString *)url :(NSString *)cust_id;

+ (void)setNavigationController:(id)navigationcontroller;
+ (id)getNavigationController;
+ (id)getRootViewController;
+ (void)startup;
+ (void)launchStage:(NSString*)stageid;
+ (void)handleGray:(NSString *)userid;
+ (void)grayChoice:(NSString *)x;
+ (long)refreshRepo:(NSString*)repoid;
+ (long)refreshCentral;

//持久化存储（文件存储）
+ (void)secureSetString:(NSString*)key value:(NSString*)value;
+ (NSString*)secureGetString:(NSString*)key;
+ (void)secureSetSm4:(NSString* )key value:(NSString* )value;
+ (NSString*)secureGetSm4:(NSString*)key;

//内存中存储（程序退出会释放）
+ (void)sessionSetString:(NSString*)key value:(NSString*)value;
+ (NSString*)sessionGetString:(NSString*)key;

/// 清除本地化持久化数据
+ (void)clearData;
/// 清除所有资源数据缓存（Cache）
+ (BOOL)clearCache;
//清除缓存（ropes）
+ (void)deleteFolder;

//用于启动资源重刷
+ (void)reloadSource;
//使用本地资源继续
+ (void)workerContinue;
//将运行日志存储在Document的local.log
+ (void)redirectLogToLocalFolder;
//自定义资源路径
+ (void)setRegistryPath:(NSString *)path;
//central备份
+ (void)setCentral_backup:(NSString *)value;
//设置链接方式（长链接、短连接）
+ (void)setConnectionKeepAlive;

//获取请求地址（以http://127.0.0.1开头的url）
// 入参path路径是类似相对路径"/xxx/xxx.png" 或者 "/xxx/xxx.do"
+ (NSString *)getMappedUrlWithPath:(NSString *)path;

//获取版本号
+(NSString *)version;
//获取Central.json对用内容
+ (NSString *)getCentralString:(NSString *)key;

/// 邦邦外发SDK-初始化  同意隐私协议后调用
/// - Parameters:
///   - keyStr: 标识KEY
///   - enableLog: Bangcle日志开关 默认关闭
+ (void)initBCESRiskStubWithKey:(NSString *)keyStr
               enableBangcleLog:(BOOL)enableLog;

/// 邦邦外发SDK-注册数据回调
/// - Parameter listener: messageinfo
+ (void)registerBCESRiskStubServiceWithDataBlock:(void(^)(NSDictionary * _Nullable dictionary, BCESRiskStubServiceCheckType modularType))listener;

/// 邦邦外发SDK-自定义弹框
/// - Parameter handler: alert
+ (void)registerBCESRiskStubAlertActionHandler:(void(^)(NSString *message, NSString *title, BCESRiskStubServiceAction action))handler;

/// 邦邦外发SDK-获取设备UDID
+ (NSString *)getBCESRiskStubEveriskUdid;
/// 邦邦外发SDK-获取版本号
+ (NSString *)getBCESRiskStubEveriskVersion;

/// 邦邦外发SDK-绑定 客户端接收用户绑定信息接口。如有特殊需求，需服务端进行定制化开发后方可使用
/// - Parameter value: 绑定信息
+ (void)addBCESRiskStubExtraUserData:(NSString *)value;

/**
 * WorkerRequest 发送请求接口
 */
+ (WorkerRequest*)newRequest:(NSString*)url method:(NSString*)method;

/**
 * AddHeader 添加header
 */
+ (void)addHeader:(NSString*)key value:(NSString*)value WorkerRequest:(WorkerRequest *)request;
/**
 * Send 发送请求若传入host地址为127.0.0.1则走worker内部的central配置根据映射去进行加密，
否则直接发送请求(在后管中配置的超时时间、ca绑定等依然有效)
 */
+ (WorkerApiResponse*)send:(NSData*)data WorkerRequest:(WorkerRequest *)request;
/**
 * SetHeader 以<key,value>的形式设置header
 */
+ (void)setHeader:(NSString*)key value:(NSString*)value WorkerRequest:(WorkerRequest *)request;
/**
 * 设置超时时间
 */
+ (void)setTimeout:(NSString*)time WorkerRequest:(WorkerRequest *)request;
/**
 * WriteField 添加表格字段
 */
+ (void)writeField:(NSString*)key value:(NSString*)value WorkerRequest:(WorkerRequest *)request;
/**
 * WriteFormFile 添加表格文件
 */
+ (BOOL)writeFormFile:(NSString*)fieldName fileName:(NSString*)fileName data:(NSData*)data error:(NSError**)error WorkerRequest:(WorkerRequest *)request;


/**
 * ResourceApi 资源代理接口
 */
+ (WorkerApiResponse*)resourceApi:(NSString*)url method:(NSString*)method body:(NSString*)body header:(NSString*)header redirectSwitch:(BOOL)redirectSwitch;
/**
 * GetBody 以字符数组的方式获取body
 */
+ (NSData*)getBody:(WorkerApiResponse *)response;
/**
 * GetCode 获取响应状态码，若响应为空则返回500
 */
+ (NSString*)getCode:(WorkerApiResponse *)response;
/**
 * GetHeader 若响应不为空，则将header进行序列化，转为json后返回
 */
+ (NSString*)getHeader:(WorkerApiResponse *)response;
/**
 * HeaderGet 根据传入的key，获取对应的header中的值
 */
+ (NSString*)headerGet:(NSString*)key WorkerApiResponse:(WorkerApiResponse *)response;
/**
 * SetHeader 以<key,value>的形式设置header
 */
+ (void)setHeader:(NSString*)key value:(NSString*)value WorkerApiResponse:(WorkerApiResponse *)response;
/**
 * String 将body转为string返回
 */
+ (NSString*)string:(WorkerApiResponse *)response;


/// 开始记录进入埋点功能时间
+ (void)startPointShowTime;

/// 获取记录进入埋点功能时间差
+ (NSString *_Nullable)getPointShowTime;
@end
